@@ -1,42 +1,49 @@
 /*
- * chntpw.c - Offline Password Edit Utility for NT 3.51 4.0 5.0 5.1 SAM database.
- * 1999-feb: Now able to browse registry hives. (write support to come)
- * 2000-jan: Attempt to detect and disable syskey
- * 2000-jun: syskey disable works on NT4. Not properly on NT5.
- * 2000-jun: changing passwords regardless of syskey.
- * 2001-jan: patched & changed to use OpenSSL. Thanks to Denis Ducamp
- * 2001-jul: extra blank password logic (when NT or LANMAN hash missing)
- * 2002-dec: New option: blank the pass (zero hash lengths).
- * 2002-dec: New option: Specify user using RID
- * 2003-jan: Support in ntreg for adding keys etc. Editor updated.
- * 2003-jan: Changed to use more of struct based V + some small stuff
- * 2004-jan: Changed some of the verbose/debug stuff
+ * chntpw.c - Offline Password Edit Utility for NT 3.51 4.0 5.0 5.1 6.0 SAM database.
+ *
+ * This program uses the "ntreg" library to load and access the registry, it's purpose
+ * is to reset password based information.
+ * There is also a simple commandline based registry editor included.
+ * 
+ * 2007-sep: Group handling extended, promotion now public
+ * 2007-sep: User edit menu, some changes to user info edit
+ * 2007-apr-may: Get and display users group memberships
+ * 2007-apr: GNU license. Some bugfixes. Cleaned up some output.
  * 2004-aug: More stuff in regedit. Stringinput bugfixes.
- * 
- * Copyright (c) 1997-2004 Petter Nordahl-Hagen.
- * Freely distributable in source or binary for noncommercial purposes,
- * but I allow some exceptions to this.
- * Please see the COPYING file for more details on
- * copyrights & credits.
- * 
- * Part of some routines, information and ideas taken from
- * pwdump by Jeremy Allison.
+ * 2004-jan: Changed some of the verbose/debug stuff
+ * 2003-jan: Changed to use more of struct based V + some small stuff
+ * 2003-jan: Support in ntreg for adding keys etc. Editor updated.
+ * 2002-dec: New option: Specify user using RID
+ * 2002-dec: New option: blank the pass (zero hash lengths).
+ * 2001-jul: extra blank password logic (when NT or LANMAN hash missing)
+ * 2001-jan: patched & changed to use OpenSSL. Thanks to Denis Ducamp
+ * 2000-jun: changing passwords regardless of syskey.
+ * 2000-jun: syskey disable works on NT4. Not properly on NT5.
+ * 2000-jan: Attempt to detect and disable syskey
+ * 1999-feb: Now able to browse registry hives. (write support to come)
+ * See HISTORY.txt for more detailed info on history.
  *
- * Some stuff from NTCrack by Jonathan Wilkins.
- * 
- *  
- * THIS SOFTWARE IS PROVIDED BY PETTER NORDAHL-HAGEN `AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ *****
  *
+ * Copyright (c) 1997-2007 Petter Nordahl-Hagen.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * See file GPL.txt for the full license.
+ * 
+ *****
+ *
+ * Information and ideas taken from pwdump by Jeremy Allison.
+ *
+ * More info from NTCrack by Jonathan Wilkins.
+ * 
  */ 
 
 #include <stdio.h>
@@ -58,7 +65,7 @@
 #include "ntreg.h"
 #include "sam.h"
 
-const char chntpw_version[] = "chntpw version 0.99.3 040818, (c) Petter N Hagen";
+const char chntpw_version[] = "chntpw version 0.99.5 070923 (decade), (c) Petter N Hagen";
 
 extern char *val_types[REG_MAX+1];
 
@@ -211,9 +218,6 @@ void check_get_samdata(void)
     
   }
 }
-    
-
-  
 
 
 /* Try to decode and possibly change account lockout etc
@@ -228,7 +232,6 @@ short handle_F(int rid, int mode)
 
   struct user_F *f;
   char s[200];
-  char yn[10];
   struct keyval *v;
   unsigned short acb;
   int b;
@@ -243,8 +246,8 @@ short handle_F(int rid, int mode)
     return(0);
   }
 
-  if (v->len != 0x50) {
-    printf("handle_F: F value is 0x%x bytes, not 0x50, unable to check account flags!\n",v->len);
+  if (v->len < 0x48) {
+    printf("handle_F: F value is 0x%x bytes, need >= 0x48, unable to check account flags!\n",v->len);
     FREE(v);
     return(0);
   }
@@ -252,7 +255,7 @@ short handle_F(int rid, int mode)
   f = (struct user_F *)&v->data;
   acb = f->ACB_bits;
 
-  if (mode) {
+  if (mode == 1) {
     printf("Account bits: 0x%04x =\n",acb);
 
 
@@ -264,30 +267,108 @@ short handle_F(int rid, int mode)
 
     printf("\nFailed login count: %u, while max tries is: %u\n",f->failedcnt,max_sam_lock);
     printf("Total  login count: %u\n",f->logins);
+  }
     
-    if (mode > 1) {
-      if ( acb & (ACB_DISABLED|ACB_AUTOLOCK) || (f->failedcnt > 0 && f->failedcnt >= max_sam_lock)  ) {
-	printf("Account is %s\n",(acb & ACB_DISABLED) ? "disabled" : "probably locked out!");
-	printf("Do you wish me to reset the failed count, unset disabled and lockout,\n");
-	fmyinput("and set the \"password never expires\" option? (y/n) [n]",yn,2);
-	if (*yn == 'y') {
-	  acb |= ACB_PWNOEXP;
-	  acb &= ~ACB_DISABLED;
-	  acb &= ~ACB_AUTOLOCK;
-	  f->ACB_bits = acb;
-	  f->failedcnt = 0;
-	  put_buf2val(hive[H_SAM], v, 0, s, REG_BINARY);
-	  printf("Unlocked!\n");
-	}
-      }
-    }
+  if (mode == 2) {
+    acb |= ACB_PWNOEXP;
+    acb &= ~ACB_DISABLED;
+    acb &= ~ACB_AUTOLOCK;
+    f->ACB_bits = acb;
+    f->failedcnt = 0;
+    put_buf2val(hive[H_SAM], v, 0, s, REG_BINARY);
+    printf("Unlocked!\n");
   }
   return (acb | ( (f->failedcnt > 0 && f->failedcnt >= max_sam_lock)<<15 ) | (acb & ACB_AUTOLOCK)<<15 | (acb & ACB_DISABLED)<<15);
 }
 
+
+/* List users membership or check if admin (is in admin group)
+ * rid   - users rid
+ * check - if 1 just check if admin, do not list
+ * returns true if user is admin
+ */
+
+int list_user_groups(int rid, int check)
+{
+  char s[200];
+  char g[200];
+  char groupname[128];
+  int nk = 0;
+  struct keyval *m = NULL, *c = NULL;
+  struct group_C *cd;
+  unsigned int *grps;
+  int count = 0, isadmin = 0;
+  int i, size, grp, grpnamoffs, grpnamlen;
+
+  if (!rid || (H_SAM < 0)) return(0);
+  
+  
+  /* Get member list for user. Go for the first full SID, it's usually local computer I hope */
+  snprintf(s,180,"\\SAM\\Domains\\Builtin\\Aliases\\Members\\S-1-5-21-\\%08X",rid);
+  /* Now, the TYPE field is the number of groups the user is member of */
+  /* Don't we just love the inconsistent use of fields!! */
+  nk = trav_path(hive[H_SAM], 0, s, 0);
+  if (!nk) {
+    /* This probably means user is not in any group. Seems to be the case
+       for a couple of XPs built in support / guest users. So just return */
+    if (gverbose) printf("list_user_groups(): Cannot find RID under computer SID <%s>\n",s);
+    return(0);
+  }
+  nk += 4;
+  count = get_val_type(hive[H_SAM],nk,"@");
+  if (count == -1) {
+    printf("list_user_groups(): Cannot find value <%s\\@>\n",s);
+    return(0);
+  }
+
+  if (!check) printf("User is member of %d groups:\n",count);
+  
+  /* This is the data size */
+  size = get_val_len(hive[H_SAM],nk,"@");
+  
+  /* It should be 4 bytes for each group */
+  if (gverbose) printf("Data size %d bytes.\n",size);
+  if (size != count * 4) {
+    printf("list_user_groups(): DEBUG: Size is not 4 * count! May not matter anyway. Continuing..\n");
+  }
+  
+  m = get_val2buf(hive[H_SAM], NULL, nk, "@", 0);
+  if (!m) {
+    printf("list_user_groups(): Could not get value data! Giving up.\n");
+    return(0);
+  }
+  
+  grps = (unsigned int *)&m->data;
+  for (i = 0; i < count; i++) {
+    grp = grps[i];
+    if (!check) printf("%08x ",grp);
+
+    if (grp == 0x220) isadmin = 1;
+
+    if (!check) {
+      snprintf(g,180,"\\SAM\\Domains\\Builtin\\Aliases\\%08X\\C",grp);
+      c = get_val2buf(hive[H_SAM], NULL, 0, g, 0);
+      if (c) {
+	cd = (struct group_C *)&c->data;
+	grpnamoffs = cd->grpname_ofs + 0x34;
+	grpnamlen  = cd->grpname_len;
+	
+	cheap_uni2ascii((char *)cd + grpnamoffs, groupname, grpnamlen);
+	
+	printf("= %s (which has %d members)\n",groupname,cd->grp_members);
+	
+      } else {
+	printf("Group info for %x not found!\n",grp);
+      }
+    }
+  }
+  return(isadmin);
+}
+
+
+
 /* Promote user into administrators group (group ID 0x220)
  * And remove from all others...
- * hdesc - hive
  * rid   - users rid
  * no returns yet
  * THIS IS VERY HACKISH YET
@@ -399,12 +480,12 @@ char *change_pw(char *buf, int rid, int vlen, int stat)
 {
    
    uchar x1[] = {0x4B,0x47,0x53,0x21,0x40,0x23,0x24,0x25};
+   char yn[4];
    int pl;
    char *vp;
    static char username[128],fullname[128];
    char comment[128],homedir[128],md4[32],lanman[32];
    char newunipw[34], newp[20], despw[20], newlanpw[16], newlandes[20];
-   char yn[4];
    int username_offset,username_len;
    int fullname_offset,fullname_len;
    int comment_offset,comment_len;
@@ -436,10 +517,15 @@ char *change_pw(char *buf, int rid, int vlen, int stat)
    ntpw_offs       = v->ntpw_ofs;
    ntpw_len        = v->ntpw_len;
 
-#if 0
-   printf("lmpw_offs: 0x%x, lmpw_len: %d (0x%x)\n",lmpw_offs,lmpw_len,lmpw_len);
-   printf("ntpw_offs: 0x%x, ntpw_len: %d (0x%x)\n",ntpw_offs,ntpw_len,ntpw_len);
-#endif
+   if (!rid) {
+     printf("No RID given. Unable to change passwords..\n");
+     return(0);
+   }
+
+   if (gverbose) {
+     printf("lmpw_offs: 0x%x, lmpw_len: %d (0x%x)\n",lmpw_offs,lmpw_len,lmpw_len);
+     printf("ntpw_offs: 0x%x, ntpw_len: %d (0x%x)\n",ntpw_offs,ntpw_len,ntpw_len);
+   }
 
    *username = 0;
    *fullname = 0;
@@ -454,7 +540,7 @@ char *change_pw(char *buf, int rid, int vlen, int stat)
       comment_offset < 0 || comment_offset >= vlen ||
       lmpw_offs < 0 || lmpw_offs >= vlen)
      {
-	if (stat != 1) printf("Not a legal struct? (negative struct lengths)\n");
+	if (stat != 1) printf("change_pw: Not a legal V struct? (negative struct lengths)\n");
 	return(NULL);
      }
 
@@ -485,21 +571,24 @@ char *change_pw(char *buf, int rid, int vlen, int stat)
 
    if (stat) {
      acb = handle_F(rid,0);
-      printf("RID: %04x, Username: <%s>%s\n",
-	     rid, username,  (  acb & 0x8000 ? ", *disabled or locked*" : (ntpw_len < 16) ? ", *BLANK password*" : "")  );
+      printf("| %04x | %-30.30s | %-6s | %-8s |\n",
+	     rid, username, (list_user_groups(rid,1) ? "ADMIN" : "") , (  acb & 0x8000 ? "dis/lock" : (ntpw_len < 16) ? "*BLANK*" : "")  );
       return(username);
    }
 
-   printf("RID     : %04d [%04x]\n",rid,rid);
+   printf("\nRID     : %04d [%04x]\n",rid,rid);
    printf("Username: %s\n",username);
    printf("fullname: %s\n",fullname);
    printf("comment : %s\n",comment);
    printf("homedir : %s\n\n",homedir);
    
-   acb = handle_F(rid,2);
+   list_user_groups(rid,0);
+   printf("\n");
 
-   if (lmpw_len < 16) {
-      printf("** LANMAN password not set. User MAY have a blank password.\n** Usually safe to continue\n");
+   acb = handle_F(rid,1);
+
+   if (lmpw_len < 16 && gverbose) {
+      printf("** LANMAN password not set. User MAY have a blank password.\n** Usually safe to continue. Normal in Vista\n");
    }
 
    if (ntpw_len < 16) {
@@ -511,20 +600,15 @@ char *change_pw(char *buf, int rid, int vlen, int stat)
 	printf("** LANMAN password IS however set. Will now install new password as NT pass instead.\n");
 	printf("** NOTE: Continue at own risk!\n");
 	ntpw_offs = lmpw_offs;
-	(unsigned int)*(vp+0xa8) = ntpw_offs - 0xcc;
+	*(vp+0xa8) = ntpw_offs - 0xcc;
 	ntpw_len = 16;
 	lmpw_len = 0;
       }
    }
-
-   if (!rid) {
-     printf("No RID given. Unable to change passwords..\n");
-     return(0);
-   }
    
    if (gverbose) {
-     hexprnt("Crypted NT pw: ",(vp+ntpw_offs),16);
-     hexprnt("Crypted LM pw: ",(vp+lmpw_offs),16);
+     hexprnt("Crypted NT pw: ",(unsigned char *)(vp+ntpw_offs),16);
+     hexprnt("Crypted LM pw: ",(unsigned char *)(vp+lmpw_offs),16);
    }
 
    /* Get the two decrpt keys. */
@@ -546,26 +630,52 @@ char *change_pw(char *buf, int rid, int vlen, int stat)
 		   (des_cblock *)&lanman[8], ks2, DES_DECRYPT);
       
    if (gverbose) {
-     hexprnt("MD4 hash     : ",md4,16);
-     hexprnt("LANMAN hash  : ",lanman,16);
+     hexprnt("MD4 hash     : ",(unsigned char *)md4,16);
+     hexprnt("LANMAN hash  : ",(unsigned char *)lanman,16);
    }
 
-   printf("\n* = blank the password (This may work better than setting a new password!)\n");
-   /*   printf("@ = promote user to administrator\n"); */
-   printf("Enter nothing to leave it unchanged\n");
-   pl = fmyinput("Please enter new password: ",newp,16);
-   
-   /*   printf("password: [%s] have length %d\n",newp,pl); */
+   printf("\n- - - - User Edit Menu:\n");
+   printf(" 1 - Clear (blank) user password\n"
+	  " 2 - Edit (set new) user password (careful with this on XP or Vista)\n"
+	  " 3 - Promote user (make user an administrator)\n");
+   printf("%s4 - Unlock and enable user account%s\n", (acb & 0x8000) ? " " : "(", 
+	  (acb & 0x8000) ? " [probably locked now]" : ") [seems unlocked already]");
+   printf(" q - Quit editing user, back to user select\n");
 
-   if (!pl) { printf("Nothing changed.\n"); return(0); }
-   
-   if (pl == 1 && *newp == '*') {
-     printf("Blanking password!\n");
-   } else if (pl == 1 && *newp == '@') {
-     promote_user(rid);
-     printf("Changes saved!\n");
+   pl = fmyinput("Select: [q] > ",newp,16);
+
+   if ( (pl < 1) || (*newp == 'q') || (*newp == 'Q')) return(0);
+
+   if (*newp == '3') {
+     printf("NOTE: This function is still experimental, and in some cases it\n"
+	    "      may result in stangeness when editing user/group in windows.\n"
+	    "      Also, users (like Guest often is) may still be prevented\n"
+	    "      from login via security/group policies which is not changed.\n");
+     fmyinput("Do you still want to promote the user? (y/n) [n] ",yn,2);
+     if (*yn == 'y' || *yn == 'Y') {
+       promote_user(rid);
+     }
      return(username);
-   } else {
+   }
+
+   if (*newp == '4') {
+     acb = handle_F(rid,2);
+     return(username);
+   }
+
+   if (*newp == '2') {
+
+     if (dontchange) {
+       printf("Sorry, unable to edit since password seems blank already (thus no space for it)\n");
+       return(0);
+     }
+
+     pl = fmyinput("New Password: ",newp,16);
+
+     if (pl < 1) {
+       printf("No change.\n");
+       return(0);
+     }
 
      cheap_ascii2uni(newp,newunipw,pl);
    
@@ -579,10 +689,10 @@ char *change_pw(char *buf, int rid, int vlen, int stat)
      
      if (gverbose) hexprnt("\nNEW MD4 hash    : ",digest,16);
      
-     E1(newlanpw,   x1, lanman);
-     E1(newlanpw+7, x1, lanman+8);
+     E1((uchar *)newlanpw,   x1, (uchar *)lanman);
+     E1((uchar *)newlanpw+7, x1, (uchar *)lanman+8);
      
-     if (gverbose) hexprnt("NEW LANMAN hash : ",lanman,16);
+     if (gverbose) hexprnt("NEW LANMAN hash : ",(unsigned char *)lanman,16);
      
      /* Encrypt the NT md4 password hash as two 8 byte blocks. */
      des_ecb_encrypt((des_cblock *)digest,
@@ -596,56 +706,48 @@ char *change_pw(char *buf, int rid, int vlen, int stat)
 		     (des_cblock *)&newlandes[8], ks2, DES_ENCRYPT);
      
      if (gverbose) {
-       hexprnt("NEW DES crypt   : ",despw,16);
-       hexprnt("NEW LANMAN crypt: ",newlandes,16);
+       hexprnt("NEW DES crypt   : ",(unsigned char *)despw,16);
+       hexprnt("NEW LANMAN crypt: ",(unsigned char *)newlandes,16);
      }
 
-   } /* blankit check */
-
-   fmyinput("\nDo you really wish to change it? (y/n) [n] ",yn,2);
-
-   if (*yn == 'y') {
-     if (pl == 1 && *newp == '*') {
-       /* Setting hash lengths to zero seems to make NT think it is blank
-        * However, since we cant cut the previous hash bytes out of the V value
-	* due to missing resize-support of values, it may leak about 40 bytes
-        * each time we do this.
-	*/
-       v->ntpw_len = 0;
-       v->lmpw_len = 0;
-     } else if (pl > 1 && *newp != '@') {
-       if (!dontchange) {
-	 /* Reset hash length to 16 if syskey enabled, this will cause
-	  * a conversion to syskey-hashes upon next boot */
-	 if (syskeyreset && ntpw_len > 16) { 
-	   ntpw_len = 16;
-	   lmpw_len = 16;
-	   ntpw_offs -= 4;
-	   (unsigned int)*(vp+0xa8) = ntpw_offs - 0xcc;
-	   *(vp + 0xa0) = 16;
-	   *(vp + 0xac) = 16;
-	 }
-	 
-	 for (i = 0; i < 16; i++) {
-	   (unsigned char)*(vp+ntpw_offs+i) = despw[i];
-	   if (lmpw_len >= 16) (unsigned char)*(vp+lmpw_offs+i) = newlandes[i];
-	 }
-       } else {
-	 printf("Unable to set since it is blank.\n");
-       }
+     /* Reset hash length to 16 if syskey enabled, this will cause
+      * a conversion to syskey-hashes upon next boot */
+     if (syskeyreset && ntpw_len > 16) { 
+       ntpw_len = 16;
+       lmpw_len = 16;
+       ntpw_offs -= 4;
+       *(vp+0xa8) = (unsigned int)(ntpw_offs - 0xcc);
+       *(vp + 0xa0) = 16;
+       *(vp + 0xac) = 16;
+     }
+     
+     for (i = 0; i < 16; i++) {
+       *(vp+ntpw_offs+i) = (unsigned char)despw[i];
+       if (lmpw_len >= 16) *(vp+lmpw_offs+i) = (unsigned char)newlandes[i];
      }
 
-#if 0
-      hexprnt("Pw in buffer: ",(vp+ntpw_offs),16);
-      hexprnt("Lm in buffer: ",(vp+lmpw_offs),16);
-#endif
-      dirty = 1;
-      printf("Changed!\n");
-   } else {
-      printf("Password not changed.\n");
+     printf("Password changed!\n");
+
+
+   } /* new password */
+
+   else if (pl == 1 && *newp == '1') {
+     /* Setting hash lengths to zero seems to make NT think it is blank
+      * However, since we cant cut the previous hash bytes out of the V value
+      * due to missing resize-support of values, it may leak about 40 bytes
+      * each time we do this.
+      */
+     v->ntpw_len = 0;
+     v->lmpw_len = 0;
+
+     printf("Password cleared!\n");
    }
    
-   printf("\n");
+#if 0
+   hexprnt("Pw in buffer: ",(vp+ntpw_offs),16);
+   hexprnt("Lm in buffer: ",(vp+lmpw_offs),16);
+#endif
+   dirty = 1;
    return(username);
 }
 
@@ -930,8 +1032,8 @@ void edit_val(struct hive *h, int nkofs, char *path)
  */
 int parsecmd(char **s, struct cmds *cmd)
 {
-  /* char temp[10]; */
-  int /* i, */ l = 0;
+
+  int l = 0;
 
   while ((*s)[l] && ((*s)[l] != ' ')) {
     l++;
@@ -983,7 +1085,8 @@ void mainloop(void)
     freetest = find_free_blk(hdesc,pagestart,10);
     printf("find_free_blk: 0x%x\n",freetest);
 #endif
-    printf("\n[%0x] %s> ",cdofs,path);
+    if (gverbose) printf("\n[%0x] %s> ",cdofs,path);
+    else printf("\n%s> ",path);
     l = fmyinput("",inbuf,90);
     bp = inbuf;
     skipspace(&bp);
@@ -998,7 +1101,7 @@ void mainloop(void)
   /* printf("pw | passwd [<hexaddr>] - try the password routine on struct at <hexaddr>\n"); */
 	printf("ck [<keyname>]          - Show keys class data, if it has any\n");
 	printf("nk <keyname> - add key\n");
-	printf("dk <keyname> - delete key (must be empty. recursion not supported yet)\n");
+	printf("dk <keyname> - delete key (must be empty)\n");
 	printf("ed <value>            - Edit value\n");
 	printf("nv <type> <valuename> - Add value\n");
 	printf("dv <valuename>        - Delete value\n");
@@ -1152,10 +1255,12 @@ int list_users(int pageit)
   if (H_SAM < 0) return(1);
   nkofs = trav_path(hive[H_SAM], 0,"\\SAM\\Domains\\Account\\Users\\Names\\",0);
   if (!nkofs) {
-    printf("Cannot find usernames in registry! (is this a SAM-hive?)\n");
+    printf("list_users: Cannot find usernames in registry! (is this a SAM-hive?)\n");
     return(1);
   }
 
+  printf("| RID -|---------- Username ------------| Admin? |- Lock? --|\n");
+
   while ((ex_next_n(hive[H_SAM], nkofs+4, &count, &countri, &ex) > 0)) {
 
     /* Extract the value out of the username-key, value is RID  */
@@ -1174,7 +1279,7 @@ int list_users(int pageit)
     }
     
     if (v->len < 0xcc) {
-      printf("Value <%s> is too short (only %d bytes) to be a SAM user V-struct!\n",
+      printf("list_users: Value <%s> is too short (only %d bytes) to be a SAM user V-struct!\n",
 	     s, v->len);
     } else {
       change_pw( (char *)&v->data , rid, v->len, 1);
@@ -1431,13 +1536,14 @@ void useredit(void)
     return;
   }
 
+
   list_users(1);
   
   while (1) {
     printf("\nSelect: ! - quit, . - list users, 0x<RID> - User with RID (hex)\n");
     printf("or simply enter the username to change: [%s] ",admuser);
     il = fmyinput("",iwho,32);
-    if (il == 1 && *iwho == '.') { list_users(1); continue; }
+    if (il == 1 && *iwho == '.') { printf("\n"); list_users(1); continue; }
     if (il == 1 && *iwho == '!') return;
     if (il == 0) strcpy(iwho,admuser);
     find_n_change(iwho);
@@ -1456,7 +1562,7 @@ void recoveryconsole()
   char yn[5];
 
   if (H_SOF < 0) {
-    printf("\nERROR: SOFTWARE-hive not loaded, and there's where RecoveryConsole settings are..\n");
+    printf("\nSOFTWARE-hive not loaded, and there's where RecoveryConsole settings are..\n");
     return;
   }
 
@@ -1464,7 +1570,7 @@ void recoveryconsole()
   sec = get_dword(hive[H_SOF],0,slpath);
 
   if (cmd == -1 && sec == -1) {
-    printf("\nDid not find registry entries for RecoveryConsole. Strange unless this is pre-win2k\n");
+    printf("\nDid not find registry entries for RecoveryConsole.\n(RecoveryConsole is only in Windows 2000 and XP)\n");
     return;
   }
 
@@ -1522,7 +1628,7 @@ void interactive(void)
   
 
 void usage(void) {
-   printf("chntpw: change password of a user in a NT SAM file, or invoke registry editor.\n"
+   printf("chntpw: change password of a user in a NT/2k/XP/2k3/Vista SAM file, or invoke registry editor.\n"
 	  "chntpw [OPTIONS] <samfile> [systemfile] [securityfile] [otherreghive] [...]\n"
 	  " -h          This message\n"
 	  " -u <user>   Username to change, Administrator is default\n"
@@ -1534,9 +1640,8 @@ void usage(void) {
           " -v          Be a little more verbose (for debuging)\n"
 	  " -L          Write names of changed files to /tmp/changed\n"
 	  " -N          No allocation mode. Only (old style) same length overwrites possible\n"
-          "See readme file on how to extract/read/write the NT's SAM file\n"
-	  "if it's on an NTFS partition!\n"
-          "Source/binary freely distributable. See README/COPYING for details\n"
+          "See readme file on how to get to the registry files, and what they are.\n"
+          "Source/binary freely distributable under GPL v2 license. See README for details.\n"
           "NOTE: This program is somewhat hackish! You are on your own!\n"
 	  );
 }
@@ -1544,8 +1649,7 @@ void usage(void) {
 int main(int argc, char **argv)
 {
    
-   /* int vofs; */
-   int dodebug = 0, list = 1, inter = 0,edit = 0,il,d = 0, dd = 0, logchange = 0, noalloc = 0;
+   int dodebug = 0, list = 2, inter = 0,edit = 0,il,d = 0, dd = 0, logchange = 0, mode = 0;
    extern int /* opterr, */ optind;
    extern char* optarg;
    char *filename,c;
@@ -1561,12 +1665,12 @@ int main(int argc, char **argv)
        case 'd': dodebug = 1; break;
        case 'e': edit = 1; break;
        case 'L': logchange = 1; break;
-       case 'N': noalloc = HMODE_NOALLOC; break;
+       case 'N': mode |= HMODE_NOALLOC; break;
        case 'l': list = 1; who = 0; break;
-       case 't': list = 2; who = 0; break;
-       case 'v': gverbose = 1; break;
-       case 'i': list = 1; who = 0; inter = 1; break;
-       case 'u': who = optarg; list = 1; break;
+       case 't': list = 3; who = 0; mode |= HMODE_TRACE; break;
+       case 'v': mode |= HMODE_VERBOSE; gverbose = 1; break;
+       case 'i': list = 2; who = 0; inter = 1; break;
+       case 'u': who = optarg; list = 2; break;
        case 'h': usage(); exit(0); break;
        default: usage(); exit(1); break;
       }
@@ -1577,7 +1681,7 @@ int main(int argc, char **argv)
    }
    do {
      if (!(hive[no_hives] = openHive(filename,
-				     HMODE_RW|noalloc|(list == 2 ? HMODE_VERBOSE : 0)))) {
+				     HMODE_RW|mode))) {
        printf("Unable to open/read a hive, exiting..\n");
        exit(1);
      }
@@ -1604,40 +1708,42 @@ int main(int argc, char **argv)
      if (inter) interactive();
    }
    
-   printf("\nHives that have changed:\n #  Name\n");
-   for (il = 0; il < no_hives; il++) {
-     if (hive[il]->state & HMODE_DIRTY) {
-       if (!logchange) printf("%2d  <%s>\n",il,hive[il]->filename);
-       d = 1;
-     }
-   }
-   if (d) {
-     /* Only prompt user if logging of changed files has not been set */
-     /* Thus we assume confirmations are done externally if they ask for a list of changes */
-     if (!logchange) fmyinput("Write hive files? (y/n) [n] : ",iwho,3);
-     if (*iwho == 'y' || logchange) {
-       if (logchange) {
-	 ch = fopen("/tmp/changed","w");
+   if (list != 1) {
+     printf("\nHives that have changed:\n #  Name\n");
+     for (il = 0; il < no_hives; il++) {
+       if (hive[il]->state & HMODE_DIRTY) {
+	 if (!logchange) printf("%2d  <%s>\n",il,hive[il]->filename);
+	 d = 1;
        }
-       for (il = 0; il < no_hives; il++) {
-	 if (hive[il]->state & HMODE_DIRTY) {
-	   printf("%2d  <%s> - ",il,hive[il]->filename);
-	   if (!writeHive(hive[il])) {
-	     printf("OK\n");
-	     if (logchange) fprintf(ch,"%s ",hive[il]->filename);
-	     dd = 2;
+     }
+     if (d) {
+       /* Only prompt user if logging of changed files has not been set */
+       /* Thus we assume confirmations are done externally if they ask for a list of changes */
+       if (!logchange) fmyinput("Write hive files? (y/n) [n] : ",iwho,3);
+       if (*iwho == 'y' || logchange) {
+	 if (logchange) {
+	   ch = fopen("/tmp/changed","w");
+	 }
+	 for (il = 0; il < no_hives; il++) {
+	   if (hive[il]->state & HMODE_DIRTY) {
+	     printf("%2d  <%s> - ",il,hive[il]->filename);
+	     if (!writeHive(hive[il])) {
+	       printf("OK\n");
+	       if (logchange) fprintf(ch,"%s ",hive[il]->filename);
+	       dd = 2;
+	     }
 	   }
 	 }
-       }
-       if (logchange) {
-	 fprintf(ch,"\n");
-	 fclose(ch);
+	 if (logchange) {
+	   fprintf(ch,"\n");
+	   fclose(ch);
+	 }
+       } else {
+	 printf("Not written!\n\n");
        }
      } else {
-       printf("Not written!\n\n");
+       printf("None!\n\n");
      }
-   } else {
-     printf("None!\n\n");
-   }
+   } /* list only check */
    return(dd);
 }
