@@ -1,6 +1,8 @@
 /*
  * ntreg.c - NT Registry Hive access library
  *
+ * 2007-sep: Verbosity/debug messages minor changes
+ * 2007-apr: LGPL license.
  * 2004-aug: Deep indirect index support. NT351 support. Recursive delete.
  *           Debugged a lot in allocation routines. Still no expansion.
  * 2004-jan: Verbosity updates
@@ -11,24 +13,22 @@
  * 
  * NOTE: The API is not frozen. It can and will change every release.
  *
- * Copyright (c) 1997-2004 Petter Nordahl-Hagen.
- * Freely distributable in source or binary for noncommercial purposes,
- * but I allow some exceptions to this.
- * Please see the COPYING file for more details on
- * copyrights & credits.
- *  
- * THIS SOFTWARE IS PROVIDED BY PETTER NORDAHL-HAGEN `AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ *****
  *
+ * NTREG - Window registry file reader / writer library
+ * Copyright (c) 1997-2007 Petter Nordahl-Hagen.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation;
+ * version 2.1 of the License.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * See file LGPL.txt for the full license.
+ * 
  */ 
 
 #include <stdio.h>
@@ -47,9 +47,9 @@
 #define DOCORE 0
 
 
-const char ntreg_version[] = "ntreg lib routines, v0.92 040818, (c) Petter N Hagen";
+const char ntreg_version[] = "ntreg lib routines, v0.94 070923, (c) Petter N Hagen";
 
-char *val_types[REG_MAX+1] = {
+const char *val_types[REG_MAX+1] = {
   "REG_NONE", "REG_SZ", "REG_EXPAND_SZ", "REG_BINARY", "REG_DWORD",       /* 0 - 4 */
   "REG_DWORD_BIG_ENDIAN", "REG_LINK",                                     /* 5 - 6 */
   "REG_MULTI_SZ", "REG_RESOUCE_LIST", "REG_FULL_RES_DESC", "REG_RES_REQ", /* 7 - 10 */
@@ -378,6 +378,7 @@ void parse_nk(struct hive *hdesc, int vofs, int blen)
 
   printf("== nk at offset %0x\n",vofs);
 
+  /* #define D_OFFS2(o) ( (void *)&(key->o)-(void *)hdesc->buffer-vofs ) */
 #define D_OFFS(o) ( (void *)&(key->o)-(void *)hdesc->buffer-vofs )
 
   key = (struct nk_key *)(hdesc->buffer + vofs);
@@ -518,7 +519,7 @@ void parse_li(struct hive *hdesc, int vofs, int blen)
 
   printf("== li at offset %0x\n",vofs);
 
-#define D_OFFS(o) ( (void *)&(key->o)-(void *)hdesc->buffer-vofs )
+  /* #define D_OFFS(o) ( (void *)&(key->o)-(void *)hdesc->buffer-vofs ) */
 
   key = (struct li_key *)(hdesc->buffer + vofs);
   printf("%04x   number of keys    = %d\n", D_OFFS(no_keys), key->no_keys  );
@@ -543,7 +544,7 @@ void parse_ri(struct hive *hdesc, int vofs, int blen)
 
   printf("== ri at offset %0x\n",vofs);
 
-#define D_OFFS(o) ( (void *)&(key->o)-(void *)hdesc->buffer-vofs )
+  /* #define D_OFFS(o) ( (void *)&(key->o)-(void *)hdesc->buffer-vofs ) */
 
   key = (struct ri_key *)(hdesc->buffer + vofs);
   printf("%04x   number of subindices = %d\n", D_OFFS(no_lis), key->no_lis  );
@@ -867,7 +868,7 @@ int alloc_block(struct hive *hdesc, int ofs, int size)
  * Will CHANGE HIVE IF SUCCESSFUL (changes linkage)
  */
 
-#define FB_DEBUG 1
+#define FB_DEBUG 0
 
 int free_block(struct hive *hdesc, int blk)
 {
@@ -1381,7 +1382,7 @@ void nk_ls(struct hive *hdesc, char *path, int vofs, int type)
   nkofs += 4;
 
   key = (struct nk_key *)(hdesc->buffer + nkofs);
-  printf("ls of node at offset 0x%0x\n",nkofs);
+  VERBF(hdesc,"ls of node at offset 0x%0x\n",nkofs);
 
   if (key->id != 0x6b6e) {
     printf("Error: Not a 'nk' node!\n");
@@ -1395,19 +1396,24 @@ void nk_ls(struct hive *hdesc, char *path, int vofs, int type)
   printf("\n");
 
   if (key->no_subkeys) {
-    printf("offs          key name\n");
+    printf("  key name\n");
     while ((ex_next_n(hdesc, nkofs, &count, &countri, &ex) > 0)) {
-      printf("[%6x]   %c  <%s>\n", ex.nkoffs, (ex.nk->len_classnam)?'*':' ',ex.name);
+      if (!(hdesc->state & HMODE_VERBOSE)) printf("%c <%s>\n", (ex.nk->len_classnam)?'*':' ',ex.name);
+      else printf("[%6x] %c <%s>\n", ex.nkoffs, (ex.nk->len_classnam)?'*':' ',ex.name);
       FREE(ex.name);
     }
   }
   count = 0;
   if (key->no_values) {
-    printf("offs        size      type   value name                    [value if type DWORD]\n");
+    printf("  size     type            value name             [value if type DWORD]\n");
     while ((ex_next_v(hdesc, nkofs, &count, &vex) > 0)) {
-      printf("[%6x] %6d  %-16s  <%s>", vex.vkoffs, vex.size,
+      if (hdesc->state & HMODE_VERBOSE) printf("[%6x] %6d  %-16s  <%s>", vex.vkoffs, vex.size,
+					       (vex.type < REG_MAX ? val_types[vex.type] : "(unknown)"), vex.name);
+      else
+      printf("%6d  %-16s  <%s>", vex.size,
 	     (vex.type < REG_MAX ? val_types[vex.type] : "(unknown)"), vex.name);
-      if (vex.type == REG_DWORD) printf(" %*d [0x%x]",25-strlen(vex.name),vex.val , vex.val);
+
+      if (vex.type == REG_DWORD) printf(" %*d [0x%x]",25-(int)strlen(vex.name),vex.val , vex.val);
       printf("\n");
       FREE(vex.name);
     }
@@ -1864,7 +1870,7 @@ int del_value(struct hive *hdesc, int nkofs, char *name)
  * return: ptr to new keystruct, or NULL
  */
 
-#undef AKDEBUG
+#define AKDEBUG 1
 struct nk_key *add_key(struct hive *hdesc, int nkofs, char *name)
 {
 
@@ -1928,7 +1934,7 @@ struct nk_key *add_key(struct hive *hdesc, int nkofs, char *name)
       oldlf = (struct lf_key *)(hdesc->buffer + oldlfofs + 0x1004);
 
 #ifdef AKDEBUG
-      printf("add_key: top of ri-loop: rislot = %d\n",rislot);
+      printf("add_key: top of ri-loop: rislot = %d, rimax = %d\n",rislot,rimax);
 #endif
       slot = -1;
 
@@ -1948,7 +1954,7 @@ struct nk_key *add_key(struct hive *hdesc, int nkofs, char *name)
 	  onkofs = oldli->hash[o].ofs_nk;
 	  onk = (struct nk_key *)(onkofs + hdesc->buffer + 0x1004);
 	  if (slot == -1) {
-#if 0
+#if 1
 	    printf("add_key: cmp <%s> with <%s>\n",name,onk->keyname);
 #endif
 
@@ -1998,7 +2004,7 @@ struct nk_key *add_key(struct hive *hdesc, int nkofs, char *name)
 	      FREE(newlf);
 	      return(NULL);
 	    }
-	    if ( cmp < 0) {
+	    if ( cmp < 0 ) {
 	      slot = o;
 	      rimax = rislot;  /* Cause end of 'ri' search, too */
 	      n++;
@@ -2017,8 +2023,7 @@ struct nk_key *add_key(struct hive *hdesc, int nkofs, char *name)
       } /* li else check */
 
 
-
-    } while ( (rislot < rimax) );  /* 'ri' wrapper loop */
+    } while ( (rislot < rimax) && (rimax > 0));  /* 'ri' wrapper loop */
 
   } else { /* Parent was empty, make new index block */
 #ifdef AKDEBUG
@@ -2395,9 +2400,9 @@ void rdel_keys(struct hive *hdesc, char *path, int vofs)
   nkofs += 4;
 
   key = (struct nk_key *)(hdesc->buffer + nkofs);
-#if 0
-  printf("rdel of node at offset 0x%0x\n",nkofs);
-#endif
+
+  VERBF(hdesc,"rdel of node at offset 0x%0x\n",nkofs);
+
   if (key->id != 0x6b6e) {
     printf("Error: Not a 'nk' node!\n");
 
@@ -2514,7 +2519,7 @@ int put_dword(struct hive *hdesc, int vofs, char *path, int dword)
   ALLOC(kr,1,sizeof(int)+sizeof(int));
   
   kr->len = sizeof(int);
-  (int)kr->data = dword;
+  kr->data = dword;
 
   r = put_buf2val(hdesc, kr, vofs, path, REG_DWORD);
 
@@ -2583,6 +2588,7 @@ struct hive *openHive(char *filename, int mode)
   struct nk_key *nk;
   struct ri_key *rikey;
   int verbose = (mode & HMODE_VERBOSE);
+  int trace   = (mode & HMODE_TRACE);
 
   CREATE(hdesc,struct hive,1);
 
@@ -2641,7 +2647,7 @@ struct hive *openHive(char *filename, int mode)
      printf("openHive(%s): File does not seem to be a registry hive!\n",filename);
      return(hdesc);
    }
-   printf("Hive's name (from header): <");
+   printf("Hive <%s> name (from header): <",filename);
    for (c = hdr->name; *c && (c < hdr->name + 64); c += 2) putchar(*c);
 
    hdesc->rootofs = hdr->ofs_rootkey + 0x1000;
@@ -2677,27 +2683,27 @@ struct hive *openHive(char *filename, int mode)
 
    while (pofs < hdesc->size) {
 #ifdef LOAD_DEBUG
-          if (verbose) hexdump(hdesc->buffer,pofs,pofs+0x20,1);
+          if (trace) hexdump(hdesc->buffer,pofs,pofs+0x20,1);
 #endif
      p = (struct hbin_page *)(hdesc->buffer + pofs);
      if (p->id != 0x6E696268) {
-       printf("Page at 0x%lx is not 'hbin', assuming file contains garbage at end",pofs);
+       printf("Page at 0x%lx is not 'hbin', assuming file contains garbage at end\n",pofs);
        break;
      }
      hdesc->pages++;
 #ifdef LOAD_DEBUG
-     if (verbose) printf("\n###### Page at 0x%0lx has size 0x%0lx, next at 0x%0lx ######\n",pofs,p->len_page,p->ofs_next);
+     if (trace) printf("\n###### Page at 0x%0lx has size 0x%0lx, next at 0x%0lx ######\n",pofs,p->len_page,p->ofs_next);
 #endif
      if (p->ofs_next == 0) {
 #ifdef LOAD_DEBUG
-       if (verbose) printf("openhive debug: bailing out.. pagesize zero!\n");
+       if (trace) printf("openhive debug: bailing out.. pagesize zero!\n");
 #endif
        return(hdesc);
      }
 #if 0
      if (p->len_page != p->ofs_next) {
 #ifdef LOAD_DEBUG
-       if (verbose) printf("openhive debug: len & ofs not same. HASTA!\n");
+       if (trace) printf("openhive debug: len & ofs not same. HASTA!\n");
 #endif
        exit(0);
      }
@@ -2706,15 +2712,15 @@ struct hive *openHive(char *filename, int mode)
 
      vofs = pofs + 0x20; /* Skip page header */
 #if 1
-     while (vofs-pofs < p->ofs_next) {
-       vofs += parse_block(hdesc,vofs,verbose);
+     while (vofs-pofs < p->ofs_next && vofs < hdesc->size) {
+       vofs += parse_block(hdesc,vofs,trace);
 
      }
 #endif
      pofs += p->ofs_next;
    }
-   printf("\nFile size %d [%x] bytes, containing %d pages (+ 1 headerpage)\n",hdesc->size,hdesc->size, hdesc->pages);
-   printf("Used for data: %d/%d blocks/bytes, unused: %d/%d blocks/bytes.\n",
+   printf("File size %d [%x] bytes, containing %d pages (+ 1 headerpage)\n",hdesc->size,hdesc->size, hdesc->pages);
+   printf("Used for data: %d/%d blocks/bytes, unused: %d/%d blocks/bytes.\n\n",
 	  hdesc->useblk,hdesc->usetot,hdesc->unuseblk,hdesc->unusetot);
   
 
@@ -2724,11 +2730,8 @@ struct hive *openHive(char *filename, int mode)
    if (trav_path(hdesc, 0, "\\SAM", 0)) hdesc->type = HTYPE_SAM;
    else if (trav_path(hdesc, 0, "\\ControlSet", 0)) hdesc->type = HTYPE_SYSTEM;
    else if (trav_path(hdesc, 0, "\\Policy", 0)) hdesc->type = HTYPE_SECURITY;
-   else if (trav_path(hdesc, 0, "\\Microsoft", 0)) hdesc->type = HTYPE_SOFTWARE;
-
-   /*
-   printf("DEBUG: Type of hive guessed to be: %d\n",hdesc->type);
-   */
+   else if (trav_path(hdesc, 0, "\\Microsoft", 0)) hdesc->type = HTYPE_SOFTWARE;   
+   if (verbose) printf("Type of hive guessed to be: %d\n",hdesc->type);
 
   return(hdesc);
 
