@@ -3,24 +3,22 @@
  * 
  * NOTE: defines are not frozen. It can and will change every release.
  *
- * Copyright (c) 1997-2003 Petter Nordahl-Hagen.
- * Freely distributable in source or binary for noncommercial purposes,
- * but I allow some exceptions to this.
- * Please see the COPYING file for more details on
- * copyrights & credits.
- *  
- * THIS SOFTWARE IS PROVIDED BY PETTER NORDAHL-HAGEN `AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ *****
  *
+ * NTREG - Window registry file reader / writer library
+ * Copyright (c) 1997-2007 Petter Nordahl-Hagen.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation;
+ * version 2.1 of the License.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * See file LGPL.txt for the full license.
+ * 
  */ 
 
 
@@ -273,6 +271,7 @@ struct keyvala {
 #define HMODE_DIRTY     0x4
 #define HMODE_NOALLOC   0x8
 #define HMODE_VERBOSE 0x1000
+#define HMODE_TRACE   0x2000
 
 /* Suggested type of hive loaded, guessed by library, but not used by it */
 #define HTYPE_UNKNOWN   0
@@ -321,8 +320,18 @@ struct hive {
     }
 #define FREE(p) { if (p) { free(p); (p) = 0; } }
 
+/* Debug / verbosity message macro */
+
+#define VERB(h, string) \
+     { \
+       if ((h)->state & HMODE_VERBOSE) printf((string)); \
+     }
+
+#define VERBF(h, string, param) \
+     { \
+       if ((h)->state & HMODE_VERBOSE) printf((string), (param)); \
+     }
 
-#endif
 
 /******* Function prototypes **********/
 
@@ -365,3 +374,6 @@ struct nk_key *add_key(struct hive *hdesc, int nkofs, char *name);
 int del_key(struct hive *hdesc, int nkofs, char *name);
 void rdel_keys(struct hive *hdesc, char *path, int nkofs);
 struct keyval *get_class(struct hive *hdesc, int curnk, char *path);
+
+#endif
+
