@@ -1,23 +1,21 @@
 /*
  * sam.h - known structures in the SAM hive of NT registry
  * 
- * Copyright (c) 1997-2004 Petter Nordahl-Hagen.
- * Freely distributable in source or binary for noncommercial purposes,
- * but I allow some exceptions to this.
- * Please see the COPYING file for more details on
- * copyrights & credits.
- *  
- * THIS SOFTWARE IS PROVIDED BY PETTER NORDAHL-HAGEN `AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
+ *****
+ *
+ * NTREG - Window registry file reader / writer library
+ * Copyright (c) 1997-2007 Petter Nordahl-Hagen.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation;
+ * version 2.1 of the License.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * See file LGPL.txt for the full license.
  *
  */ 
 
@@ -185,5 +183,37 @@ struct user_V {
 
 };
 
+/* Groups C data struct
+ * First 0x34 bytes is pointer & len table, rest is data which
+ * the table points to
+ * String data is unicode, not zero terminated (must use len)
+ */
+
+struct group_C {
+
+  int groupid;         /* 0x00 - Seems to be the group ID */
+  int unknown1_1;      /* 0x04 - always zero? */
+  int unknown1_2;      /* 0x08 - points to groupname? */
+  int unknown1_3;      /* 0x0c - always 0x02 0x00 0x01 0x00 ? */
+
+  int grpname_ofs;    /* 0x10 */
+  int grpname_len;    /* 0x14 */
+
+  int unknown2_1;      /* 0x18 - always zero? */
+
+  int fullname_ofs;    /* 0x1c */
+  int fullname_len;    /* 0x20 */
+
+  int unknown3_1;      /* 0x24 - always zero? */
+  int unknown3_2;      /* 0x28 - always zero? */
+  int unknown3_3;      /* 0x2c - always zero? */
+
+  int grp_members;     /* 0x30 - number of group members */
+
+  /* 0x34 - data starts here. pointers above are offset from this */
+
+  char data[];
+
+};
 
 #endif
